'use strict';
angular.module('aom.services')
  .factory('aomDrawingSrv', function ($q, $timeout, aomFileSystemSrv, aomJobSrv, aomAttachmentsSrv, aomLogSrv, aomPopupSrv, aomResourcesSrv, aomComponentsSrv) {
    var drawing = {};
    var history = [];
    var future = [];
    var now = null;
    var canvas;
    var loading;
    var changed;
    var controlTimeout = null;

    function addToHistory() {
      if (!loading && controlTimeout === null) {
        history.push(now);
        now = canvas.toJSON(['initialStrokeWidth']);
        future.length = 0;
      }
    }

    function loadCanvasFromJSON(canvasJson) {
      var def = $q.defer();
      loading = true;
      canvas.loadFromJSON(canvasJson, function () {
        canvas.renderAll();
        now = JSON.stringify(canvas);
        loading = false;
        def.resolve();
      });
      return def.promise;
    }

    function setSelectedObject() {
      if (controlTimeout === null) {
        var object = canvas.getActiveObject();
        var group = canvas.getActiveGroup();
        if (object && object.type !== 'rect') {
          object.lockUniScaling = true;
        }
        if (object) {
          object.cornerSize = 25;
          object.transparentCorners = true;
          object.initialWidth = object.width;
          object.initialHeight = object.height;
          object.initialRadius = object.radius;
          object.initialFontSize = object.fontSize;
          object.initialStrokeWidth = object.initialStrokeWidth !== undefined ? object.initialStrokeWidth : object.strokeWidth;
          object.isText = object.type === 'i-text';
          object.isFillable = object.type === 'circle' || object.type === 'rect' || object.type === 'triangle';
        }
        if (group) {
          group.cornerSize = 25;
          group.transparentCorners = true;
          group.lockUniScaling = true;
          group.lockScalingX = true;
          group.lockScalingY = true;
        }
        $timeout(function () {
          drawing.data.selectedObject = object;
        });
        $timeout(function () {
          drawing.data.selectedGroup = group;
        });
      } else {
        $timeout(function () {
          setSelectedObject();
        }, 50);
      }
    }

    function setLastObjectActive() {
      canvas.setActiveObject(canvas.getObjects()[canvas.getObjects().length - 1]);
      setSelectedObject();
      canvas.renderAll();
    }

    function scaling() {
      if (controlTimeout !== null) {
        $timeout.cancel(controlTimeout);
      }
      var obj = canvas.getActiveObject();
      controlTimeout = $timeout(function () {
        var width = obj.initialWidth * obj.scaleX,
          height = obj.initialHeight * obj.scaleY,
          radius = obj.initialRadius * obj.scaleX,
          fontSize = obj.initialFontSize * obj.scaleX;

        if (!obj.path) {
          obj.set({
            initialRadius: radius,
            initialHeight: height,
            initialWidth: width,
            initialFontSize: fontSize,
            radius: radius,
            height: height,
            width: width,
            fontSize: fontSize,
            scaleX: 1,
            scaleY: 1
          });
        } else {
          obj.set({
            strokeWidth: parseInt(obj.initialStrokeWidth / obj.scaleX, 10)
          });
        }

        canvas.renderAll();
        controlTimeout = null;
        addToHistory();
      }, 300);
    }

    function addObject(object) {
      object.top = (canvas.height / 2) - 30;
      object.left = (canvas.width / 2) - 30;
      canvas.add(object);
      drawing.setMode(false);
      setLastObjectActive();
    }

    function addShape(object) {
      object.fill = 'transparent';
      object.stroke = canvas.freeDrawingBrush.color;
      object.strokeWidth = canvas.freeDrawingBrush.width;
      addObject(object);
    }

    drawing.resetDrawingSurface = function () {
      drawing.data = {
        selectedObject: null,
        deselectMethod: null,
        selectMethod: null,
        selectedGroup: null
      };
    };

    drawing.init = function (ctx, backgroundImage, mappedProps, field) {

      changed = false;
      drawing.isPhotoTaken = false;
      var headerFooterHeight = 120;

      var drawingFieldElement = document.getElementById('drawing-field');
      $timeout(function () {
        // ctx.canvas.width = window.innerHeight > window.innerWidth ? window.innerHeight : window.innerWidth;
        // ctx.canvas.height = window.innerHeight < window.innerWidth ? window.innerHeight - headerFooterHeight : window.innerWidth - headerFooterHeight;
        const statusBarHeight = ionic.Platform.isIOS() ? 20 : 0;
        ctx.canvas.width = drawingFieldElement.clientWidth;
        ctx.canvas.height = drawingFieldElement.clientHeight - 44 - 44 - statusBarHeight;

        canvas = new fabric.Canvas('c', {
          isDrawingMode: true
        });

        canvas.objectCaching = false;

        canvas.on('object:modified', addToHistory);

        canvas.on('object:added', addToHistory);

        canvas.on('object:removed', addToHistory);

        canvas.on('object:scaling', scaling);

        canvas.on('object:selected', function () {
          if (drawing.data.selectMethod) {
            drawing.data.selectMethod();
          }
          setSelectedObject();
        });

        canvas.on('selection:cleared', function () {
          setSelectedObject();
          if (drawing.data.deselectMethod) {
            drawing.data.deselectMethod();
          }
        });

        if (backgroundImage !== null) {
          if (field.originalAttachment === undefined && field.MaintainBackgroundImageOnClear) {
            field.originalAttachment = field.Attachments.length === 1 ? field.Attachments[0] : 0;
          }
          drawing.setBackground(backgroundImage);
        }

        fabric.Object.prototype.transparentCorners = false;

        if (canvas.freeDrawingBrush) {
          canvas.freeDrawingBrush.color = mappedProps.drawingColour;
          canvas.freeDrawingBrush.width = parseInt(mappedProps.lineWidth, 10) || 1;
          canvas.freeDrawingBrush.shadowBlur = 0;
        }

        now = JSON.stringify(canvas);
        history.length = 0;
        future.length = 0;
        $timeout(function () {
          drawing.forceAddToHistory();
        }, 300);
      });
    };

    drawing.setBackground = function (backgroundImage) {
      new fabric.Image.fromURL(
        window.cordova ? aomFileSystemSrv.jobDataPathNormalized + backgroundImage : backgroundImage,
        function (img) {
          var renderOptions = {};
          if (img.height * img.scaleToWidth(canvas.width).scaleX > canvas.height) {
            img.scaleToHeight(canvas.height);
            renderOptions.left = (canvas.width - (img.width * img.scaleX)) / 2;
          } else {
            img.scaleToWidth(canvas.width);
            renderOptions.top = (canvas.height - (img.height * img.scaleX)) / 2;
          }

          canvas.setBackgroundImage(img, canvas.renderAll.bind(canvas), renderOptions);
        });
    };

    drawing.addImage = function (image) {
      new fabric.Image.fromURL(
        window.cordova ? aomFileSystemSrv.jobDataPathNormalized + image : image,
        function (img) {
          img.scaleToHeight(canvas.height - 50);
          img.top = 25;
          img.left = 25;
          canvas.add(img);
          setLastObjectActive();
        });
    };

    drawing.setMode = function (drawing) {
      canvas.isDrawingMode = drawing;
      canvas.deactivateAll();
      setSelectedObject();
      canvas.renderAll();
    };

    drawing.clearClick = function (field) {
      changed = true;
      canvas.clear();
      field.Data = '';
      if (field.originalAttachment !== undefined && field.originalAttachment !== 0) {
        drawing.setBackground(field.originalAttachment.filename);
      } else {
        canvas.backgroundImage = 0;
        canvas.renderAll();
      }
      addToHistory();
    };

    drawing.selectLineWidth = function (lineWidth) {
      canvas.freeDrawingBrush.width = parseInt(lineWidth, 10) || 1;
    };

    drawing.setObjectLineWidth = function (lineWidth) {
      if (controlTimeout !== null) {
        $timeout.cancel(controlTimeout);
      }
      controlTimeout = $timeout(function () {
        canvas.getActiveObject().strokeWidth = parseInt(lineWidth, 10);
        canvas.renderAll();
        controlTimeout = null;
      }, 300);
    };

    drawing.colorSelected = function (selectedColor) {
      if (canvas.freeDrawingBrush) {
        canvas.freeDrawingBrush.color = selectedColor;
      }
    };

    drawing.setObjectColor = function (selectedColor, fill) {
      if (controlTimeout !== null) {
        $timeout.cancel(controlTimeout);
      }
      controlTimeout = $timeout(function () {
        var obj = canvas.getActiveObject();
        if (obj.isText) {
          obj.set({ fill: selectedColor });
        }
        if (fill) {
          obj.set({ fill: selectedColor });
        } else {
          obj.set({ stroke: selectedColor });
        }
        canvas.renderAll();
        controlTimeout = null;
      }, 300);
    };

    drawing.undo = function () {
      if (history.length > 1) {
        var poppedHistory = history.pop();
        future.push(now);
        loadCanvasFromJSON(poppedHistory);
      }
    };

    drawing.redo = function () {
      if (future.length > 0) {
        var poppedFuture = future.pop();
        history.push(now);
        loadCanvasFromJSON(poppedFuture);
      }
    };

    drawing.remove = function () {
      if (canvas.getActiveGroup()) {
        canvas.getActiveGroup().forEachObject(function (o) { canvas.remove(o); });
        canvas.discardActiveGroup().renderAll();
      } else {
        canvas.remove(canvas.getActiveObject());
      }
    };

    drawing.scale = function (factor) {
      if (canvas.getActiveObject() !== null) {
        var obj = canvas.getActiveObject();
        if (!obj.path) {
          obj.width = (parseInt(factor) / 10) * obj.initialWidth;
          obj.height = (parseInt(factor) / 10) * obj.initialHeight;
          obj.radius = (parseInt(factor) / 10) * obj.initialRadius;
          obj.fontSize = (parseInt(factor) / 10) * obj.initialFontSize;
        } else {
          obj.scaleX = (parseInt(factor) / 10);
          obj.scaleY = (parseInt(factor) / 10);
          obj.strokeWidth = parseInt(obj.initialStrokeWidth / obj.scaleX, 10);
        }
        canvas.renderAll();
      }
    };

    drawing.addCircle = function () {
      var circle = new fabric.Circle({
        radius: 30
      });
      addShape(circle);
    };

    drawing.addSquare = function () {
      var square = new fabric.Rect({
        width: 60,
        height: 60
      });
      addShape(square);
    };

    drawing.addTriangle = function () {
      var triangle = new fabric.Triangle({
        width: 60,
        height: 60
      });
      addShape(triangle);
    };

    drawing.addText = function (data) {
      var text = new fabric.IText(data, {
        fontFamily: 'Helvetica',
        fill: canvas.freeDrawingBrush.color,
        lineHeight: canvas.freeDrawingBrush.width / 10,
        editable: false
      });
      addObject(text);
    };

    drawing.updateText = function (data) {
      drawing.data.selectedObject.text = data;
      canvas.renderAll();
    };

    drawing.deselectObject = function () {
      canvas.deactivateAll().renderAll();
    };

    drawing.saveCanvas = function (field, comment) {
      canvas.deactivateAll();
      canvas.renderAll();
      var fileName = field.Name + moment().format('HHmmssSS') + '.png';
      var file = { data: aomComponentsSrv.dataURLtoBlob(canvas.toDataURL()) };
      var firstAttachment = '';
      var operation = $q.when;
      if (field.Attachments.length > 0) {
        firstAttachment = field.Attachments[0].filename;
        operation = field.Attachments[0].Source === undefined ? aomAttachmentsSrv.removeInMemoryAttachment : aomAttachmentsSrv.deleteAttachment;
      }
      aomAttachmentsSrv.data.attachments = field.Attachments;
      return operation(firstAttachment)
        .then(function () {
          return aomFileSystemSrv.write(fileName, file);
        })
        .then(function () {
          canvas.width = 0;
          canvas.height = 0;
          canvas = null;
          return aomAttachmentsSrv.addAttachment(aomJobSrv.data.selectedJob.Id, { name: fileName, type: '.png' }, comment, 'Drawing');
        })
        .catch(function (err) {
          aomPopupSrv.alert('DrawingSaveFailed', {
            title: aomResourcesSrv.data.resources.Common.Error,
            template: aomResourcesSrv.data.resources.ImageEditor.FailedToSaveImage,
            onDismiss: function () { }
          });
          aomLogSrv.log({ message: 'Failed to save drawing.', error: err }, aomLogSrv.levels.user, aomLogSrv.severities.warn);
        });
    };

    drawing.changesMade = function () {
      return canvas._objects.length > 0 || drawing.isPhotoTaken || changed;
    };

    drawing.forceAddToHistory = addToHistory;

    return drawing;

  });
