'use strict';
angular.module('aom.services')
  .factory('aomLocationSrv', function ($q, $cordovaGeolocation, aomResourcesSrv, aomAPISrv, aomLogSrv) {
    var aomLocationSrv = {},
      lastFetchAttempt = null;

    aomLocationSrv.data = {
      locationRetrieved: window.cordova ? false : true,
      lastKnownLocation: {
        latitude: 0,
        longitude: 0
      },
      lastLocationForServer: {
        latitude: null,
        longitude: null
      }
    };

    function setFakeLocation() {
      //Set office address
      var latitude = 51.284313;
      var longitude = -0.5950213;
      aomLocationSrv.data.lastKnownLocation.latitude = latitude;
      aomLocationSrv.data.lastKnownLocation.longitude = longitude;
      aomLocationSrv.data.lastLocationForServer = { latitude: latitude, longitude: longitude };
      aomLocationSrv.data.locationRetrieved = true;
    }

    aomLocationSrv.getAccurateCurrentLocation = function (options) {

      var def = $q.defer();
      var bestCheckedPosition,
          locationEventCount = 0,
          watchID,
          timerID;

      options = options || {};

      var updateLastKnownLocation = function(position) {
        lastFetchAttempt = new Date();
        aomLocationSrv.data.lastKnownLocation.latitude = position.coords.latitude;
        aomLocationSrv.data.lastKnownLocation.longitude = position.coords.longitude;
        aomLocationSrv.data.lastLocationForServer = position.coords;
        aomLocationSrv.data.locationRetrieved = true;
      }

      var setNoLastKnownLocation = function(err) {
        lastFetchAttempt = new Date();
        aomLocationSrv.data.lastLocationForServer = { latitude: 0, longitude: 0 };
        aomLogSrv.log({ message: 'Failed to get location', error: err }, aomLogSrv.levels.low, aomLogSrv.severities.err);
      }

      var checkLocation = function (position) {
        locationEventCount += 1;
        if (locationEventCount >= options.countMin) {
          if (!bestCheckedPosition || position.coords.accuracy <= bestCheckedPosition.coords.accuracy) {
            bestCheckedPosition = position;
          }
          if (position.coords.accuracy <= options.desiredAccuracy) {
              clearTimeout(timerID);
              navigator.geolocation.clearWatch(watchID);
              updateLastKnownLocation(bestCheckedPosition);
              def.resolve();
          }
        }
      };

      var stopTrying = function () {
        navigator.geolocation.clearWatch(watchID);
        if (bestCheckedPosition) {
          updateLastKnownLocation(bestCheckedPosition);
        }
        else {
          setNoLastKnownLocation({ code:3, message:'Timeout after trying for ' + options.maxWait + ' ms' });
        }
        def.resolve();
      };

      var onError = function (error) {
          clearTimeout(timerID);
          navigator.geolocation.clearWatch(watchID);
          setNoLastKnownLocation(error);
          def.resolve();
      };

      // On iOS the returned accuracy was large even though the latitude and longitude were actually fairly precise
      // Setting to 100 eliminates the wildly inacurrate results e.g.: 1500+ without restricting iOS locations
      // The most important part of this code is the minimum count i.e.: the first gps position is always bad

      if (isNaN(options.maxWait)) options.maxWait = 10000; // Default 10 seconds
      if (isNaN(options.desiredAccuracy)) options.desiredAccuracy = 100; // Default 100 metres accuracy    
      if (isNaN(options.timeout)) options.timeout = options.maxWait; // Default to maxWait
      if (isNaN(options.maximumAge)) options.maximumAge = 0; // Default current locations only
      if (isNaN(options.countMin)) options.countMin = 2; // Default ignore first event because some devices send a cached value even with max age 0
                                                            
      options.enableHighAccuracy = true; // Force high accuracy (otherwise, why are you using this function?)
      watchID = navigator.geolocation.watchPosition(checkLocation, onError, options); // Watch the current gps position
      timerID = setTimeout(stopTrying, options.maxWait); // Set a timeout that will abandon the location loop

      return def.promise;

    };

    function getCurrentPosition() {
      return $cordovaGeolocation.getCurrentPosition({
        timeout: 10000,
        maximumAge: 0,
        enableHighAccuracy: true
      });
    }

    aomLocationSrv.getCurrentLocation = function (useCache = false) {
      if (!window.cordova) {
        setFakeLocation();
        return $q.when();
      } else {
        var now = new Date();
        if (!useCache || (!lastFetchAttempt || (now - lastFetchAttempt > 30000))) {
          return getCurrentPosition()
            .then(function (position) {
              lastFetchAttempt = now;
              aomLocationSrv.data.lastKnownLocation.latitude = position.coords.latitude;
              aomLocationSrv.data.lastKnownLocation.longitude = position.coords.longitude;
              aomLocationSrv.data.lastLocationForServer = position.coords;
              aomLocationSrv.data.locationRetrieved = true;
            })
            .catch(function (err) {
              lastFetchAttempt = now;
              aomLocationSrv.data.lastLocationForServer = { latitude: 0, longitude: 0 };
              aomLogSrv.log({ message: 'Failed to get location', error: err }, aomLogSrv.levels.low, aomLogSrv.severities.err);
              return $q.when();
            });
        } else {
          return $q.when();
        }
      }
    };

    aomLocationSrv.sendToServer = function () {
      return getCurrentPosition()
        .then(function (position) {
          var locationInfo = {
            Latitude: position.coords.latitude,
            Longitude: position.coords.longitude,
            Altitude: position.coords.altitude,
            Accuracy: position.coords.accuracy,
            AltitudeAccuracy: position.coords.altitudeAccuracy,
            Heading: position.coords.heading,
            Speed: position.coords.speed,
            Timestamp: position.timestamp
          };
          return aomAPISrv.postData(aomAPISrv.data.rootLinks.Location.href, locationInfo);
        });
    };

    return aomLocationSrv;
  });
